C    *******************************************************************
        program gcmc
	implicit none
	include 'gcmc.inc'
C    *******************************************************************

        print *, 'Gran canonical (gcmc) version 0.1'
        print *, 'Montecarlo simulation of Lennard-Jones atoms'
        print *, '***************************************************'
        print *, '* Calculo de propiedades estructurales            *'
        print *, '* para un sistema de Lennard Jones.               *'
        print *, '* Hector Dominguez castro                         *'
        print *, '* Tutorial realizado con el apoyo del proyecto    *'
        print *, '* PAPIME-DGAPA-PE107419                           *'
        print *, '***************************************************'

        call config
        call start
	call markov
	call finish
	
C    *******************  END OF THE PROGRAM ***************************

	stop
	end
	

	
	
C    *******************************************************************
        subroutine start
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    ** Local variables ************************************************
	doubleprecision dens, vol, tpot
C    *******************************************************************
	
c	print *, 'Gran canonical (gcmc) version 0.1'
c	print *, 'Montecarlo simulation of Lennard-Jones atoms'
c	print *, '***************************************************'
c        print *, '* Calculo de propiedades estructurales            *'
c        print *, '* para un sistema de Lennard Jones.               *'
c        print *, '* Hector Dominguez castro                         *'
c        print *, '* Tutorial realizado con el apoyo del proyecto    *'
c        print *, '* PAPIME-DGAPA-PE107419                           *' 
c        print *, '***************************************************'
          
        print *,'Dar el archivo para guardar la ultima configuracion'
        read  (*,'(a)') rundat
        write (*,'('' file      '',a)') rundat
        print *,'Archivo para guardar los datos g(r)'
        read  (*,'(a)') disnam
        write (*,'('' file      '',a)') disnam
        print *, 'Dar el maximo desplazamiento'
        read  *, drmax                                       
	print *, 'Numero de bloques'
	read  *, nblock
	print *, 'Numero de ciclos'
	read  *, nstep
        print *, 'Dar intervalo para actualizar max. desplazamiento'
        read  *, iratio
        print *, 'Dar la temperatura'
        read  *, temp
	print *, 'Distancia de corte para el potencial'
	read  *, rcut
        print *, 'Dar potencial quimico de exceso en terminos de B'
        print *, '--- B=mu/kt + ln(N) ---'
        read  *, chem

        ro    = box/200
        nrad  = int(anint(box/(2.0*ro)))

        print *, 'No. puntos en g(r)        = ', nrad
        print *, 'Ancho intervalos para g(r)= ', ro
        print *, 'pot. quim. exc.  B(mu)    = ', chem
	print *, 'Numbero de bloques        = ', nblock
        print *, 'Numbero de ciclos         = ', nstep
        print *, 'razon de frecuencia       = ', iratio
        print *, 'Maximo desplazamiento     = ', drmax
        print *, 'temperatura               = ', temp
        print *, 'radio de corte Potential  = ', rcut


        vol  = box*box*hlen
        dens = dble(nav)/vol
	
C    *** Initial configurational energy
	call totener

        tpot  = pot/dble(nav)
	eng   = 1.5*temp + tpot
        press = dens*temp + vir/vol
	
	print *, 'Valores iniciales'
        print *, 'No.part.ini  = ', nav
        print *, 'Pot.quim.Exc = ', chem
	print *, 'Potencial    = ', tpot
	print *, 'Total ener   = ', eng
	print *, 'Temperatura  = ', temp
        print *, 'Densidad     = ', dens
        print *, 'Presion      = ', press
	
	return
	end
	
	
C    *******************************************************************
        subroutine markov
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    ** run blocks for a move, remove or creation of one particle   ****
C    ** Local variables ************************************************
        integer         block, step, i, j, n1, n2, n3
	doubleprecision dens, acatma, beta, runnum
        doubleprecision ratio, dummy, ranf, prob, vol, tpot
C    *******************************************************************

C    ** Markov Chain **
        write(*,'(//'' start of Markov Chain     ''//)')

        vol  = box*box*hlen
        beta = 1.0/temp

C    ** Zero run accumulator **

        n1 = 0
        n2 = 0
        n3 = 0
        runpot = 0.0
        runpre = 0.0
        runden = 0.0
        runnum = 0.0
        do j=1, nrad
         runrad(j) = 0.0
        enddo

          write(*,'(1x,''cycle'',''  Potencial'',''       den'',
     :              ''    pres'',
     :              ''    totene'',''      temp'')')

        do block = 1, nblock

C    ** Zero accumulators **
            acatma = 0.0
            acp    = 0.0
            acd    = 0.0
            acpre  = 0.0

C          *** choose option for a move, destroy or creation ***
           do step = 1, nstep
             n1 = n1+1
             call mov(acatma)
             prob = ranf(dummy)
             prob = anint(prob)
             if (prob .eq. 1) then
               n2 = n2+1
               call inp
              else
               n3 = n3+1
               call out
             endif

C            *** calculate properties in the system ***********
              dens  = dble(nav)/vol
              tpot  = pot/dble(nav)
              press = dens*temp + vir/vol

              call radial

C       **   accumulate averages **             
              do j=1, nrad
                runrad(j) = runrad(j)+gdp(j)
              enddo
              acp    = acp + tpot
              acd    = acd + dens
              acpre = acpre + press
              runpot = runpot + tpot
              runden = runden + dens
              runnum = runnum + nav
              runpre = runpre + press

C       **  Perform periodic operations **
             if (mod(step,iratio) .eq. 0 ) then
              ratio = acatma/dble(nav*iratio)
              if (ratio .gt. 0.5) then
                  drmax = drmax*1.05
              else
                  drmax = drmax*0.95
              endif
              acatma = 0.0
             endif
             
	   enddo
C ****     End loop over steps

           acp   = acp / dble(nstep)
           acd   = acd / dble(nstep)
           acpre = acpre / dble(nstep)

           eng    = 1.5*temp + acp

C             write(*,'(1x,'' ** valores acomulados ** '')')
             write(*,'(i5,2x,5f10.4)') block, acp, acd, acpre,
     :             eng, temp

        enddo
C ****   End loop over blocks

C       ** Normalize run accumulator **      
           do j=1, nrad
             runrad(j) = runrad(j)/dble(nstep*nblock)
           enddo
           runpot = runpot/dble(nstep*nblock)
           runden = runden/dble(nstep*nblock)
           runnum = runnum/dble(nstep*nblock)
           runpre = runpre/dble(nstep*nblock)
           eng    = 1.5*temp + runpot

           write(*,'(1x,'' ** valores ** '')')
            write(*,'(1x,5x,5f10.4)') runpot, runden, runpre,
     :             eng, temp

C         *** print number of attempts the system did **********
          print *,'mov = ', n1
          print *,'inp = ', n2
          print *,'out = ', n3
          print *,'numero promedio de particulas = ', anint(runnum)
               
        return
        end
          
C       *** Ends the loop over cycles ***
C       *** End of Markov Chain ****


C    *******************************************************************
        subroutine finish
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    ** Local variables ************************************************
	integer         i, j, loci
        doubleprecision dens, vol, fdp(200), tpot
C    *******************************************************************

         vol  = box*box*hlen
         dens = dble(nav)/vol

C   *** average of the pair correlation function ***
        do j=1, nrad
           fdp(j) = runrad(j)
        enddo

C   *** final total configurational energy
        call totener

        tpot  = pot/dble(nav)
        eng   = 1.5*temp + tpot
        press = dens*temp + vir/vol

	print *, 'Valores Finales'
        print *, 'No.part.fin  = ', nav
        print *, 'Pot.Quim.Exc.= ', chem
	print *, 'Potencial    = ', tpot
	print *, 'Total ener   = ', eng
	print *, 'Temperatura  = ', temp
        print *, 'Densidad     = ', dens
        print *, 'presion      = ', press
c        do loci = 1, nav
c           i = locate(loci)
c        print *,'x,y,z=',rx(i), ry(i), rz(i)
c        enddo

c         do loci=1, nav
c          i = locate(loci)
c          rz(i) = rz(i) + hlen/2.0d0
c         enddo

	
C    *** keep data in files *****************
	  open(unit=3,file=rundat,status='unknown')
	  write(3,'(1x,i5)') nav
	  write(3,'(1x,3f10.5)') box, box, hlen
          do loci = 1, nav
           i = locate(loci)
	   write(3,'(1x,3f10.5)') rx(i), ry(i), rz(i)
	  enddo
	  close(unit=3)

c        print *,'Real thickness in distr. func.    = ', ro
        open(unit=3,file=disnam,status='unknown')
C        write(3,'(1x,''   k'',''   r'',''   g(2)'')')
        do i=1, nrad
          write(3,'(1x,i5,2f10.5)') i,i*ro-ro/2,fdp(i)
        enddo
        close(unit=3)


        return
	end


C    *******************************************************************
        subroutine mov(acatma)
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    ** attempt a trial move of one particle                       *****
C    ** Local variables ************************************************
        integer         i, nloc
	doubleprecision deltv, deltvi, deltvb, acatma
        doubleprecision dummy, ranf, vnew, vold, virnew, virold, beta
        doubleprecision rxiold, ryiold, rziold, rxinew, ryinew, rzinew 
        doubleprecision boxinv
C    *******************************************************************

           beta   = 1.0/temp
           boxinv = 1.0/box

c          do nloc = 1, nav
             nloc = int(dble(nav-1)*ranf(dummy))+1
               i= locate(nloc)
                rxiold = rx(i)
                ryiold = ry(i)
                rziold = rz(i)

C            *** Calculate the energy of i in the old configuration ***
  
             call enegi(rxiold,ryiold,rziold,i,vold,virold)

C            *** Move i and pickup the central image ***

                rxinew = rxiold + (2.0*ranf(dummy) - 1.0)*drmax
                ryinew = ryiold + (2.0*ranf(dummy) - 1.0)*drmax
                rzinew = rziold + (2.0*ranf(dummy) - 1.0)*drmax
             
                rxinew = rxinew - box*anint(rxinew*boxinv)
                ryinew = ryinew - box*anint(ryinew*boxinv)
                rzinew = rzinew - hlen*anint(rzinew/hlen)
               
C            *** Calculate the energy of i in the new configuration ***

             call enegi(rxinew, ryinew, rzinew, i, vnew, virnew)

C            *** Check for acceptance ***
             
                deltv   = vnew - vold
                deltvi  = virnew - virold
                deltvb  = beta*deltv

                if (deltvb .lt. 75.0) then
                  if (deltvb .le. 0.0) then
                   pot    = pot + deltv
                   vir    = vir + deltvi
                   rx(i)  = rxinew 
                   ry(i)  = ryinew 
                   rz(i)  = rzinew 
                   acatma = acatma + 1.0
                  elseif (exp(-deltvb) .gt. ranf(dummy)) then
                   pot    = pot + deltv
                   vir    = vir + deltvi
                   rx(i)  = rxinew 
                   ry(i)  = ryinew 
                   rz(i)  = rzinew
                   acatma = acatma + 1.0
                  endif
                endif
c           enddo

           return
           end


C    *******************************************************************
        subroutine inp
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    ** attempt a trial creation of one particle                   *****
C    ** Local variables ************************************************
        integer         ntrial
        logical         create, ovrlap
        doubleprecision dummy, ranf, rmin, vol
        doubleprecision deltv, deltvi, deltcb, beta
        doubleprecision rxnew, rynew, rznew, ls2
C    *******************************************************************
  
        ls2    = box/2.0
        create = .false.
        beta   = 1.0/temp
        rmin   = 0.2
        ntrial = nav+1

        if (ntrial .ge. maxn) stop 'maximum number of atoms in box'
C    ** Generate the position of the trial atom **

        rxnew = box*ranf(dummy)-ls2
        rynew = box*ranf(dummy)-ls2
        rznew = hlen*ranf(dummy)-hlen/2.0
C    ** Calculete energy change on addition **

        call potin (rxnew,rynew,rznew,rmin,deltv,deltvi,ovrlap)

C    ** check for acceptance **
        vol = box*box*hlen       
        if (.not. ovrlap) then
C *** Activity in terms of excess chemical potential,Z=rho*exp(mu/kt)***
C           act = exp(chem*beta)*dble(ntrial)/vol
C *** Activity in terms of B=mu/kt + ln(N), Z=exp(B)/vol
           act = exp(chem)/vol
           deltcb = beta*deltv-log(act*vol/dble(ntrial))
           if (deltcb .lt. 75.0) then
             if (deltcb .lt. 0.0) then
                create = .true.
                call add (rxnew,rynew,rznew)
                vir    = vir + deltvi
                pot    = pot + deltv
                nav    = ntrial
             elseif (exp(-deltcb) .gt. ranf(dummy)) then
                create = .true.
                call add (rxnew,rynew,rznew)
                vir    = vir + deltvi
                pot    = pot + deltv
                nav    = ntrial
             endif
           endif
        endif
 
        return
        end
   

C    *******************************************************************
        subroutine out
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    ** attempt a trial destruction of one particle                 ****
C    ** Local variables ************************************************
        integer         ntrial, nloc, ipull
        logical         ghost
        doubleprecision dummy, ranf, vol 
        doubleprecision deltv, deltvi, deltdb, beta
C    *******************************************************************

        ghost  = .false.
        beta   = 1.0/temp
        ntrial = nav-1
        
        if (ntrial .eq. 1) stop 'only one atom remains'
C    ** pick a random element from the active part of locate **

        nloc  = int(dble(ntrial)*ranf(dummy))+1
        ipull = locate(nloc)

C    ** calculate energy change on removal of atom ipull **

        call potout (ipull,deltv,deltvi)

C    ** check for acceptance **
        vol    = box*box*hlen
C *** Activity in terms of excess chemical potential,Z=rho*exp(mu/kt)***
C        act    = exp(chem*beta)*dble(nav)/vol
C *** Activity in terms of B=mu/kt + ln(N), Z=exp(B)/vol
        act = exp(chem)/vol
        deltdb = -beta*deltv-log(dble(nav)/(vol*act))
        if (deltdb .lt. 75.0) then
           if (deltdb .lt. 0.0) then
              ghost  = .true.
              call remove (nloc,ipull)
              vir    = vir - deltvi
              pot    = pot - deltv
              nav    = ntrial
            elseif (exp(-deltdb) .gt. ranf(dummy)) then
              ghost  = .true.
              call remove (nloc,ipull)
              vir    = vir - deltvi
              pot    = pot - deltv
              nav    = ntrial
           endif
        endif

        return
        end


C    *******************************************************************
        subroutine totener
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    ** calculate total energy of the system                       *****
C    ** Local variables ************************************************
	integer         i, j, loci, jin
	doubleprecision rxi, ryi, rzi, boxinv
	doubleprecision rxij, ryij, rzij, rijsq, rcutsq, vcut
	doubleprecision r2ij, r6ij, r12ij, fij, vij, hlenin
	doubleprecision fxij, fyij, fzij
C    *******************************************************************

        rcutsq= rcut*rcut
        r2ij  = 1.0d0/rcutsq
        r6ij  = r2ij*r2ij*r2ij
        r12ij = r6ij*r6ij
        vcut  = r12ij-r6ij

        boxinv = 1.0/box
        hlenin = 1.0/hlen

C    ** Zero energies **
        pot   = 0.0d0
	vir   = 0.0d0

C    ** Outer loop over i **
        do loci = 1, nav-1
           i= locate(loci)
	   rxi = rx(i)
	   ryi = ry(i)
	   rzi = rz(i)
	   
C       ** Inner loop over j **
           do j = loci+1, nav
              jin = locate(j)
	      rxij = rxi - rx(jin)
	      ryij = ryi - ry(jin)
	      rzij = rzi - rz(jin)
	         rxij  = rxij - anint(rxij*boxinv)*box
	         ryij  = ryij - anint(ryij*boxinv)*box
                 rzij  = rzij - anint(rzij*hlenin)*hlen
		 rijsq = rxij*rxij + ryij*ryij + rzij*rzij
		 if ( rijsq .lt. rcutsq ) then
		    r2ij  = 1.0/rijsq
		    r6ij  = r2ij*r2ij*r2ij
		    r12ij = r6ij*r6ij
		    vij   = r12ij - r6ij
		    fij   = (vij + r12ij)*r2ij
                    vij   = vij - vcut
		    pot   = pot + vij
		    fxij  = fij*rxij
		    fyij  = fij*ryij
		    fzij  = fij*rzij
		    vir   = vir+rxij*fxij+ryij*fyij+rzij*fzij
		 endif
	   enddo
	enddo


C    ** Incorporate correct numerical factors **
        pot   = pot * 4.0d0
	vir   = vir * 24.0d0/3.0d0

	return
	end


C    *******************************************************************
        subroutine enegi(rxi,ryi,rzi,i,poti,viri)
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    ** calculate the change of energy for a move particle          ****
C    ** contribution from the wall are included                     ****
C    ** Local variables ************************************************
        integer         i, j, jin
	doubleprecision rxi, ryi, rzi, boxinv, hlenin, poti, viri
	doubleprecision rxij, ryij, rzij, rijsq, rcutsq, vcut
	doubleprecision r2ij, r6ij, r12ij, fij, vij
	doubleprecision fxij, fyij, fzij
C    *******************************************************************

        rcutsq= rcut*rcut
        r2ij  = 1.0/rcutsq
        r6ij  = r2ij*r2ij*r2ij
        r12ij = r6ij*r6ij
        vcut  = r12ij-r6ij

        boxinv = 1.0/box
        hlenin = 1.0/hlen

C    ** Zero energies **
        poti   = 0.0
	viri   = 0.0

C       ** Loop over all molecules except i **
           do j = 1, nav
              jin = locate(j)
              if (i .ne. jin) then
	       rxij = rxi - rx(jin)
	       ryij = ryi - ry(jin)
	       rzij = rzi - rz(jin)
	         rxij  = rxij - anint(rxij*boxinv)*box
	         ryij  = ryij - anint(ryij*boxinv)*box
                 rzij  = rzij - anint(rzij*hlenin)*hlen
		 rijsq = rxij*rxij + ryij*ryij + rzij*rzij
		 if ( rijsq .lt. rcutsq ) then
		    r2ij  = 1.0/rijsq
		    r6ij  = r2ij*r2ij*r2ij
		    r12ij = r6ij*r6ij
		    vij   = r12ij - r6ij
		    fij   = (vij + r12ij)*r2ij
c                    vij   = vij - vcut
		    poti  = poti + vij
		    fxij  = fij*rxij
		    fyij  = fij*ryij
		    fzij  = fij*rzij
		    viri  = viri+rxij*fxij+ryij*fyij+rzij*fzij
		 endif
               endif    
     	   enddo

C    ** Incorporate correct numerical factors **
        poti   = poti * 4.0
	viri   = viri * 24.0/3.0

	return
	end


C    *******************************************************************
        subroutine potin (rxi,ryi,rzi,rmin,deltv,deltvi,ovrlap)
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    ** calculate the change of energy for an add particle          ****
C    ** contribution from the wall are included                     ****
C    ** Local variables ************************************************
	integer         j, jin
	doubleprecision rxi, ryi, rzi,boxinv, hlenin
	doubleprecision rmin, deltv, deltvi, rcutsq, vcut
	doubleprecision rminsq, r2ij, r6ij, r12ij, fij, vij
	doubleprecision rxij, ryij, rzij, rijsq
        doubleprecision fxij, fyij, fzij
        logical         ovrlap
C    *******************************************************************

        ovrlap = .false.
        rminsq = rmin*rmin
        rcutsq = rcut*rcut
        r2ij  = 1.0/rcutsq
        r6ij  = r2ij*r2ij*r2ij
        r12ij = r6ij*r6ij
        vcut  = r12ij-r6ij

        boxinv = 1.0/box
        hlenin = 1.0/hlen

C    ** zero accumulators **
        deltv   = 0.0
        deltvi  = 0.0

C    ** loop over all atoms **
        
        do j =1, nav
           jin   = locate(j)
            rxij  = rxi-rx(jin)
            ryij  = ryi-ry(jin)
            rzij  = rzi-rz(jin)
              rxij  = rxij-anint(rxij*boxinv)*box
              ryij  = ryij-anint(ryij*boxinv)*box
              rzij  = rzij - anint(rzij*hlenin)*hlen
              rijsq = rxij*rxij+ryij*ryij+rzij*rzij
	      if ( rijsq .lt. rminsq ) then
                 ovrlap = .true.
                 return
               elseif (rijsq .lt. rcutsq) then
	         r2ij   = 1.0/rijsq
	         r6ij   = r2ij*r2ij*r2ij
	         r12ij  = r6ij*r6ij
                 vij    = r12ij - r6ij
                 fij    = (vij + r12ij)*r2ij
c                 vij    = vij - vcut
	         deltv  = deltv + vij
	         fxij   = fij*rxij
	         fyij   = fij*ryij
	         fzij   = fij*rzij
	         deltvi = deltvi+rxij*fxij+ryij*fyij+rzij*fzij
              endif
        enddo

C   **  incorporate numerical factor **
        deltv  = deltv*4.0
        deltvi = deltvi*24.0/3.0


        return
        end


C    *******************************************************************
        subroutine potout (ipull,deltv,deltvi)
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    ** calculate the change of energy for a remove particle        ****
C    ** contribution from the wall are included                     ****
C    ** Local variables ************************************************
	integer         j, jin, ipull
	doubleprecision rxi, ryi, rzi
	doubleprecision deltv, deltvi, rcutsq, vcut
	doubleprecision r2ij, r6ij, r12ij, fij, vij, boxinv, hlenin
	doubleprecision rxij, ryij, rzij, rijsq
        doubleprecision fxij, fyij, fzij
C    *******************************************************************

        rcutsq = rcut*rcut
        r2ij = 1.0/rcutsq
        r6ij  = r2ij*r2ij*r2ij
        r12ij = r6ij*r6ij
        vcut  = r12ij-r6ij

        boxinv = 1.0/box
        hlenin = 1.0/hlen

C    ** zero accumulators **
        deltv  = 0.0
        deltvi = 0.0
 
        rxi = rx(ipull)
        ryi = ry(ipull)
        rzi = rz(ipull)
        
C    ** loop over all atoms except ipull**
        
        do j =1, nav
           jin   = locate(j)
              if (jin .ne. ipull) then
                rxij  = rxi-rx(jin)
                ryij  = ryi-ry(jin)
                rzij  = rzi-rz(jin)
                rxij  = rxij-anint(rxij*boxinv)*box
                ryij  = ryij-anint(ryij*boxinv)*box
                rzij = rzij - anint(rzij*hlenin)*hlen
                rijsq = rxij*rxij+ryij*ryij+rzij*rzij
  	       if ( rijsq .lt. rcutsq ) then
	          r2ij   = 1.0/rijsq
	          r6ij   = r2ij*r2ij*r2ij
	          r12ij  = r6ij*r6ij
                  vij    = r12ij - r6ij
                  fij    = (vij + r12ij)*r2ij
c                  vij    = vij - vcut
	          deltv  = deltv + vij
	          fxij   = fij*rxij
	          fyij   = fij*ryij
	          fzij   = fij*rzij
	          deltvi = deltvi+rxij*fxij+ryij*fyij+rzij*fzij
                endif
              endif
        enddo

C   **  incorporate numerical factor **
        deltv  = deltv*4.0
        deltvi = deltvi*24.0/3.0

        return
        end



C    *******************************************************************
        subroutine add (rxnew, rynew, rznew)
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    *** add a particle to the array locate                  ***********
C    *** local variables ***********************************************
        integer         inew, ntrial  
        doubleprecision rxnew, rynew, rznew
C    *******************************************************************

        ntrial = nav+1
        inew = locate(ntrial)
        if (inew .eq. 0) then
C    **    atom requieres a new number **
           locate(ntrial) = ntrial
           inew           = ntrial
        endif

C    ** fit new atom into the array **

        rx(inew) = rxnew
        ry(inew) = rynew
        rz(inew) = rznew

        return
        end


C    *******************************************************************
        subroutine remove (nloc, ipull)
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    *** remove a particle from the array locate                 *******
C    *** local variables ***********************************************
        integer         ipull, nloc, k
C    *******************************************************************

        if (nloc .lt. nav) then
C    **     close up the array locate after the removal **
            do  k = nloc+1, nav
                locate(k-1) = locate(k)
            enddo
C    **     place the ghost atom ipull just outside the active **

            locate(nav) = ipull
        endif     

        return
        end



C    *******************************************************************
        subroutine radial
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    *** construct pair distribution function g(r) in each layer     ***
C    *** g(r) is in two dimension (radial in x-y direction)          ***
C    *** local variables ***********************************************
        integer         i, k, m, j, loci, jin
        doubleprecision boxinv, nideal,rlower, rupper, cte, pi
        doubleprecision xij, yij, rsq, zij, rij
        parameter       (pi=3.14159265)
C    *******************************************************************

        boxinv = 1.0/box
        maxrad =hlen/2.0
        ro     = maxrad/dble(nrad)


       do j=1, nrad
          gdp(j)  = 0.0
        enddo

        do loci=1, nav-1
          i=locate(loci)
         do jin = loci+1,nav
            j=locate(jin)
             xij = rx(i)-rx(j)
             yij = ry(i)-ry(j)
             zij = rz(i)-rz(j)
             xij = xij - box*anint(xij*boxinv)
             yij = yij - box*anint(yij*boxinv)
             zij = zij - hlen*anint(zij/hlen)
             rsq = xij*xij + yij*yij + zij*zij
             rij = sqrt(rsq)
             k   = int(rij/ro)+1
             if (k .le. nrad) then
              gdp(k) = gdp(k)+2
             endif
          enddo
         enddo
         cte = 4.0*pi*(dble(n)/(box*box*hlen))/3.0

         do m=1, nrad
          rlower = dble(m-1)*ro
          rupper = rlower+ro
          nideal = cte*(rupper**3 - rlower**3)
          gdp(m) = gdp(m)/(dble(nav)*nideal)
         enddo


        return
        end


C    *******************************************************************
        subroutine config
	implicit none
	include 'gcmc.inc'
C    *******************************************************************
C    *** local variables ***********************************************
        integer         i
        character       name*20
C    *******************************************************************

        print *,'Dar el archivo de configuracion inicial'
        read  (*,'(a)') name
        write (*,'('' file      '',a)') name
        open(unit=3,file=name, status='old')
	read(3,*) n
	read(3,*) box, box, hlen
        do i = 1, n
	   read(3,*) rx(i), ry(i), rz(i) 
	enddo
	close(unit=3)
 
        nav = n

c         do i=1, nav
c          rz(i) = rz(i) - hlen/2.0d0
c         enddo

C    *** initialize position to zero for future created particles ***
        maxn = n+5000

        do i=n+1, maxn
           rx(i)=0.0
           ry(i)=0.0
           rz(i)=0.0
        enddo

C    *** array to put particles after a created or destroyed part. ***
        do i=1, maxn
           locate(i) = i
        enddo

C     ** random number ***
         call ranset

        return
        end


C    *******************************************************************
        subroutine ranset
	implicit none
	include 'gcmc.inc'
C    ********************************************************************
C    ***** local variables *****        
        integer         m1, m2, m3, ia1, ia2, ia3, ic1, ic2, ic3
        integer         j, idum, seed
        doubleprecision rm1, rm2
        parameter       (m1=259200, ia1=7141, ic1=54773)
        parameter       (m2=134456, ia2=8121, ic2=28411)
        parameter       (m3=243000, ia3=4561, ic3=51349)
        parameter       (rm1=1.0/m1, rm2=1.0/m2)  
C    ********************************************************************

         print *, 'Dar un numero entero al azar'
         read  *, seed

        idum = 2.0*seed + 1.0
        ix1  = mod(ic1+idum,m1)
        ix1  = mod(ia1*ix1+ic1,m1)
        ix2  = mod(ix1,m2)
        ix1  = mod(ia1*ix1+ic1,m1)
        ix3  = mod(ix1,m3)  

        do j=1, 97
         ix1      = mod(ia1*ix1+ic1,m1)
         ix2      = mod(ia2*ix2+ic2,m2) 
         rando(j) = (float(ix1) + float(ix2)*rm2)*rm1
        enddo

  
        return
        end


C    *******************************************************************
        doubleprecision function ranf (dummy)
	implicit none
	include 'gcmc.inc'
C    ********************************************************************
C    ***** local variables *****        
        integer         m1, m2, m3, ia1, ia2, ia3, ic1, ic2, ic3, j
        doubleprecision rm1, rm2, dummy
        parameter       (m1=259200, ia1=7141, ic1=54773)
        parameter       (m2=134456, ia2=8121, ic2=28411)
        parameter       (m3=243000, ia3=4561, ic3=51349)
        parameter       (rm1=1.0/m1, rm2=1.0/m2)  
C    ********************************************************************
C        print *,'HOLA11'
10      ix1 = mod(ia1*ix1+ic1,m1)
        ix2 = mod(ia2*ix2+ic2,m2)
        ix3 = mod(ia3*ix3+ic3,m3)
20      j   = 1 + (97*ix3)/m3
        if ((j .gt. 97) .or. (j .lt. 1)) then
         print *, 'j error', j
         goto 20
        endif
       
        ranf     = rando(j)
        rando(j) = (float(ix1) + float(ix2)*rm2)*rm1

        if ((ranf .lt. 0.0) .or. (ranf .gt. 1)) then
         print *, 'error in random number'
         goto 10
        endif
           
        return
        end




